package com.ruoyi.app.mapper;

import com.ruoyi.app.domain.AppTask;

import java.util.List;

/**
 * 【请填写功能名称】Mapper接口
 *
 * @author ruoyi
 * @date 2025-01-15
 */
public interface AppTaskMapper
{
    /**
     * 查询【请填写功能名称】
     *
     * @param taskId 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    public AppTask selectAppTaskByTaskId(String taskId);

    /**
     * 查询【请填写功能名称】列表
     *
     * @param appTask 【请填写功能名称】
     * @return 【请填写功能名称】集合
     */
    public List<AppTask> selectAppTaskList(AppTask appTask);

    /**
     * 新增【请填写功能名称】
     *
     * @param appTask 【请填写功能名称】
     * @return 结果
     */
    public int insertAppTask(AppTask appTask);

    /**
     * 批量新增
     * @param appTaskList 新增list
     * @return 结果
     */
    int batchInsertAppTask(List<AppTask> appTaskList);

    /**
     * 修改【请填写功能名称】
     *
     * @param appTask 【请填写功能名称】
     * @return 结果
     */
    public int updateAppTask(AppTask appTask);

    /**
     * 批量更新
     * @param appTaskList 更新list
     * @return 结果
     */
    int batchUpdateAppTask(List<AppTask> appTaskList);

    /**
     * 删除【请填写功能名称】
     *
     * @param taskId 【请填写功能名称】主键
     * @return 结果
     */
    public int deleteAppTaskByTaskId(String taskId);

    /**
     * 批量删除【请填写功能名称】
     *
     * @param taskIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAppTaskByTaskIds(String[] taskIds);

    List<AppTask> selectPlcList();

    int updatePlcId(AppTask appTask);
}
